@@ -48,10 +48,10 @@ residuals.ctmm <- function(object,data,...)
 residuals.telemetry <- function(object,CTMM=NULL,...) { residuals.ctmm(CTMM,object,...) }
 
 # acf function
-correlogram <- function(data,dt=NULL,fast=TRUE,res=1,axes=c("x","y"))
+correlogram <- function(data,dt=NULL,fast=TRUE,res=1,axes=c("x","y"),trace=TRUE)
 {
   if(fast) { ACF <- variogram.fast(data,dt=dt,res=res,CI="IID",axes=axes,ACF=TRUE) }
-  else { ACF <- variogram.slow(data,dt=dt,CI="IID",axes=axes,ACF=TRUE) }
+  else { ACF <- variogram.slow(data,dt=dt,CI="IID",axes=axes,trace=trace,ACF=TRUE) }
 
   # normalize ACF to 1 at lag 0
   ACF$SVF <- ACF$SVF / ACF$SVF[1]
