@@ -1,4 +1,4 @@
-encounter <- function(object,include=NULL,exclude=NULL,debias=FALSE,...)
+cde <- function(object,include=NULL,exclude=NULL,debias=FALSE,...)
 {
   UD <- object
   check.projections(UD)
@@ -13,7 +13,7 @@ encounter <- function(object,include=NULL,exclude=NULL,debias=FALSE,...)
   CTMM <- lapply(UD,function(U){U@CTMM})
 
   # Gaussian approximation
-  STUFF <- encounter.ctmm(CTMM,include=include,exclude=exclude,debias=debias)
+  STUFF <- cde.ctmm(CTMM,include=include,exclude=exclude,debias=debias)
   CTMM <- STUFF$CTMM
   BIAS <- STUFF$BIAS # individual biases
   bias <- STUFF$bias # pairwise biases
@@ -81,7 +81,7 @@ encounter <- function(object,include=NULL,exclude=NULL,debias=FALSE,...)
   object$H <- H
   object$DOF.area <- DOF.area
 
-  info <- mean.info(UD)
+  info <- mean_info(UD)
   object <- new.UD(object,info=info,type='range',variable="encounter",CTMM=CTMM)
 
   return(object)
@@ -89,7 +89,7 @@ encounter <- function(object,include=NULL,exclude=NULL,debias=FALSE,...)
 
 ################
 # Gaussian encounters
-encounter.ctmm <- function(CTMM,include=NULL,exclude=NULL,debias=FALSE,...)
+cde.ctmm <- function(CTMM,include=NULL,exclude=NULL,debias=FALSE,...)
 {
   # check.projections(CTMM)
 
@@ -196,7 +196,7 @@ encounter.ctmm <- function(CTMM,include=NULL,exclude=NULL,debias=FALSE,...)
   NAMES <- names(sigma@par)[1:length(I)] # isotropic
   dimnames(COV) <- list(NAMES,NAMES)
 
-  info <- mean.info(CTMM)
+  info <- mean_info(CTMM)
 
   CTMM <- ctmm(mu=mu,sigma=sigma,COV.mu=COV.mu,COV=COV,axes=axes,isotropic=isotropic,info=info)
   return(list(CTMM=CTMM,BIAS=BIAS,bias=bias))
@@ -204,22 +204,32 @@ encounter.ctmm <- function(CTMM,include=NULL,exclude=NULL,debias=FALSE,...)
 
 
 # relative encounter rates
-rates <- function(object,debias=TRUE,level=0.95,normalize=TRUE,self=TRUE,...)
+encounter <- function(object,debias=FALSE,level=0.95,normalize=FALSE,self=TRUE,...)
 {
   units <- FALSE
 
   R <- overlap(object,debias=debias,level=level,method="Rate",...)
-  R <- R$CI
+  R$CI <- nant(R$CI,0)
+  R$DOF <- nant(R$DOF,0)
 
   if(normalize)
   {
-    M <- 1/diag(R[,,'est']) # TODO !!! REMOVE MEAN BIAS CORRECTION
-    M <- mean(M) # TODO !!! REPLACE THIS WITH META-MEAN
-    R <- R*M
+    # calculate mean self-rate
+    s <- diag(R$CI[,,'est'])
+    dof <- diag(R$DOF)
+    s <- meta.chisq(s,dof)$CI["mean","est"]
+
+    R$CI <- R$CI/s
   }
 
   # fix diagonals # self encounter rate
-  if(self) { diag(R[,,1]) <- diag(R[,,2]) <- diag(R[,,3]) <- Inf }
+  if(self)
+  {
+    diag(R$CI[,,1]) <- diag(R$CI[,,2]) <- diag(R$CI[,,3]) <- Inf
+    diag(R$DOF) <- Inf
+  }
+
+  R$CI <- R$CI * pi # standardized encounter probability (1-meter)
 
   return(R)
 }
