@@ -49,6 +49,8 @@ get.loglike <- function(data)
 # FIT MODEL WITH LIKELIHOOD FUNCTION (convenience wrapper to optim)
 ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),trace=FALSE)
 {
+  # check.class(data)
+
   loglike.fn <- get.loglike(data)
 
   if(!is.null(control$message)) { message <- control$message }
@@ -144,7 +146,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   COV.init <- CTMM$COV
   # make sure we can start from previous failed fit
   if(any(is.nan(COV.init) | COV.init==Inf)) { COV.init <- NULL }
-  if(!is.null(COV.init)) { TEST <- eigen(COV.init,only.values=TRUE)$values } else { TEST <- FALSE }
+  if(!is.null(COV.init)) { TEST <- eigen(COV.init)$values } else { TEST <- FALSE }
   if(any(TEST<=.Machine$double.eps | TEST==Inf)) { COV.init <- NULL }
   # erase previous fitting info if present
   CTMM$COV <- NULL
@@ -154,8 +156,17 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   # evaluate mean function and error matrices for this data once upfront
   CTMM <- ctmm.prepare(data,CTMM,tau=FALSE,calibrate=FALSE) # don't muck with taus, don't calibrate
   TYPE <- DOP.match(axes)
-  UERE.DOF <- attr(data,"UERE")$DOF[,TYPE]
-  names(UERE.DOF) <- rownames(attr(data,"UERE")$DOF)
+  if(TYPE!="unknown")
+  {
+    UERE.DOF <- attr(data,"UERE")$DOF[,TYPE]
+    names(UERE.DOF) <- rownames(attr(data,"UERE")$DOF)
+  }
+  else
+  {
+    UERE.DOF <- 0
+    names(UERE.DOF) <- "all"
+  }
+
   UERE.FIT <- CTMM$error>0 & !is.na(UERE.DOF) & UERE.DOF<Inf # will we be fitting any error parameters?
   UERE.FIX <- CTMM$error>0 & (is.na(UERE.DOF) | UERE.DOF==Inf) # are there any fixed error parameters?
   UERE.PAR <- names(UERE.FIT)[UERE.FIT>0] # names of fitted UERE parameters
@@ -163,7 +174,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   if(any(!UERE.FIT)) { CTMM$error[!UERE.FIT] <- as.logical(CTMM$error[!UERE.FIT]) }
 
   # don't try to fit error class parameters absent from data
-  if(any(CTMM$error>0) && "class" %in% names(data))
+  if(any(CTMM$error>0) && "class" %in% names(data) && TYPE!="unknown")
   {
     LEVELS <- levels(data$class)
     UERE.DOF <- UERE.DOF[LEVELS]
@@ -172,6 +183,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
     UERE.PAR <- UERE.PAR[UERE.PAR %in% LEVELS]
     CTMM$error <- CTMM$error[LEVELS]
   }
+
   # make sure to include calibration in log-likelihood even if error==FALSE
   CTMM$errors <- any(CTMM$error>0)
 
@@ -529,22 +541,33 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
 #################
 ic.ctmm <- function(CTMM,n)
 {
-  NAMES <- CTMM$features
   axes <- CTMM$axes
   range <- CTMM$range
-  k.mean <- nrow(CTMM$mu)
+  k.mean <- length(CTMM$mu)
   method <- CTMM$method
 
-  nu <- length(NAMES)
   # all parameters
   q <- length(axes)
-  if(!range)
+  if(is.null(CTMM$formula)) # autocorrelation model
   {
-    k.mean <- k.mean - 1
-    n <- n - 1
+    NAMES <- CTMM$features
+    nu <- length(NAMES)
+
+    if(!range)
+    {
+      k.mean <- k.mean - q
+      n <- n - 1
+    }
+    if(!length(k.mean)) { k.mean <- 0 } # failed fit (bad data or bad parameters)
   }
-  if(!length(k.mean)) { k.mean <- 0 } # failed fit (bad data or bad parameters)
-  k <- nu + q*k.mean
+  else # RSF model
+  {
+    k.beta <- length(all.vars(CTMM$formula)) # linear terms
+    nu.beta <- length(CTMM$beta) - k.beta # quadratic terms (and more)
+    k.mean <- 2 + k.beta
+    nu <- 1 + nu.beta
+  }
+  k <- nu + k.mean
 
   CTMM$AIC <- 2*k - 2*CTMM$loglike
   CTMM$BIC <- log(n)*k - 2*CTMM$loglike
@@ -553,9 +576,9 @@ ic.ctmm <- function(CTMM,n)
   if(method=='ML')
   { CTMM$AICc <- -2*CTMM$loglike + q*n * 2*k/(q*n-k-nu) }
   else if(method=='pREML')
-  { CTMM$AICc <- -2*CTMM$loglike + (q*n)^2/(q*n+q*k.mean) * 2*k/(q*n-k-nu) }
+  { CTMM$AICc <- -2*CTMM$loglike + (q*n)^2/(q*n+k.mean) * 2*k/(q*n-k-nu) }
   else if(method %in% c('pHREML','HREML','REML'))
-  { CTMM$AICc <- -2*CTMM$loglike + (q*n-q*k.mean) * 2*k/(q*n-k-nu) }
+  { CTMM$AICc <- -2*CTMM$loglike + (q*n-k.mean) * 2*k/(q*n-k-nu) }
 
   # fix divergence
   if(q*n<=k+nu) { CTMM$AICc <- Inf }
@@ -625,7 +648,8 @@ ic.ctmm <- function(CTMM,n)
 ###################
 ctmm.guess <- function(data,CTMM=ctmm(),variogram=NULL,name="GUESS",interactive=TRUE)
 {
-  if(class(data)[1]=="list") { stop("ctmm.guess needs to be run individually.") }
+  check.class(data)
+
   # use intended axes
   if(is.null(variogram)) { variogram = variogram(data,axes=CTMM$axes) }
   else { CTMM$axes <- attr(variogram,"info")$axes }
