@@ -51,7 +51,12 @@ meta.chisq <- function(s,dof,level=0.95,level.pop=0.95,IC="AICc",method='mle',bo
 
   # this is MVU for both chi^2 and IG
   # this is 1st order debiased in between
-  inverse.mean <- function(mu,Vm2) { 1/mu * max(1-debias*Vm2,0) }
+  inverse.mean <- function(mu,Vm2,MIN=2)
+  {
+    dof <- 2/Vm2
+    dof <- max(dof,MIN)
+    1/mu * (1-debias*2/dof)
+  }
   # Vm2 = VAR[mu]/mu^2
 
   ################
@@ -123,7 +128,8 @@ meta.chisq <- function(s,dof,level=0.95,level.pop=0.95,IC="AICc",method='mle',bo
     if(complete)
     {
       NAME[1] <- "Dirac-\u03B4"
-      mu <- sum(dof*s)/sum(dof) # exact solution
+      w <- dof/sum(dof)
+      mu <- sum(w*s) # exact solution
       mu <- nant(mu,mean(s)) # if any dof==Inf
       PAR[1,1] <- mu
       LL[1] <- -nloglike(mu)
@@ -133,8 +139,10 @@ meta.chisq <- function(s,dof,level=0.95,level.pop=0.95,IC="AICc",method='mle',bo
     NAME[2] <- "inverse-Gaussian"
     if(complete && n>=2) # par==NULL
     {
-      mu <- mean(s)
-      k <- mean(1/s) - 1/mu
+      w <- 1-exp(-dof) # turn off low dof from mean
+      w <- w/sum(w)
+      mu <- sum(w*s)
+      k <- sum(w/s) - 1/mu
       par <- c(mu,k)
     }
     if((complete && n>=2) || length(par)==2)
@@ -206,8 +214,8 @@ meta.chisq <- function(s,dof,level=0.95,level.pop=0.95,IC="AICc",method='mle',bo
 
       # inverse-mean area
       CI[2,] <- 1/CI[1,] # not used
-      CI[2,2] <- CI[2,2] * dof/max(dof-debias,0) # inverse-chi^2 mean bias correction
-      CI.VAR[2] <- 2*CI[2,2]^2/max(dof-3*debias,0)
+      CI[2,2] <- CI[2,2] * dof/max(dof-debias,1) # inverse-chi^2 mean bias correction
+      CI.VAR[2] <- 2*CI[2,2]^2/max(dof-3*debias,1)
 
       # CoV^2 (RVAR)
       CI[3,] <- c(0,0,Inf)
@@ -220,7 +228,7 @@ meta.chisq <- function(s,dof,level=0.95,level.pop=0.95,IC="AICc",method='mle',bo
     else if(IND==2) # inverse-Gaussian model
     {
       STUFF <- genD(par,nloglike,lower=c(0,0),order=2)
-      COV <- cov.loglike(STUFF$hessian,STUFF$gradient)
+      COV <- cov.loglike(STUFF$hessian,STUFF$gradient,WARN=is.na(IC))
 
       if(debias) # Bessel's correction to point estimate of k=1/lambda and COV
       {
@@ -518,6 +526,7 @@ meta <- function(x,variable="area",level=0.95,level.UD=0.95,method="MLE",IC="AIC
 ############
 import.variable <- function(x,variable="area",level.UD=0.95,chi=FALSE)
 {
+  x <- name.list(x)
   N <- length(x)
   ID <- names(x) # may be null
 
@@ -550,6 +559,15 @@ import.variable <- function(x,variable="area",level.UD=0.95,chi=FALSE)
         AREA[i] <- STUFF$CI[2]
         DOF[i] <- STUFF$DOF * 2 # 2D
       }
+      else if(variable=="kinetic")
+      {
+        STUFF <- summary(x[[i]],units=FALSE)
+        DOF[i] <- STUFF$DOF['speed'] * 2 # 2D
+        if(DOF[i]>0)
+        { AREA[i] <- STUFF$CI['speed (meters/second)','est']^2 }
+        else
+        { AREA[i] <- Inf }
+      }
       else if(variable=="tauposition")
       {
         E <- x[[i]]$tau[1]
@@ -561,7 +579,9 @@ import.variable <- function(x,variable="area",level.UD=0.95,chi=FALSE)
         else
         {
           AREA[i] <- E
-          DOF[i] <- 2*E^2/x[[i]]$COV["tau position","tau position"]
+          P <- c("tau position","tau")
+          P <- P[ P %in% rownames(x[[i]]$COV) ]
+          DOF[i] <- 2*E^2/x[[i]]$COV[P,P]
         }
       }
       else if(variable=="tauvelocity")
@@ -575,7 +595,9 @@ import.variable <- function(x,variable="area",level.UD=0.95,chi=FALSE)
         else
         {
           AREA[i] <- E
-          DOF[i] <- 2*E^2/x[[i]]$COV["tau velocity","tau velocity"]
+          P <- c("tau velocity","tau")
+          P <- P[ P %in% rownames(x[[i]]$COV) ]
+          DOF[i] <- 2*E^2/x[[i]]$COV[P,P]
         }
       }
     }
@@ -587,6 +609,12 @@ import.variable <- function(x,variable="area",level.UD=0.95,chi=FALSE)
         DOF[i] <- x[[i]]$DOF[1,2]
         AREA[i] <- -log(x[[i]]$CI[1,2,'est'])
       }
+      if(CLASS=="distance")
+      {
+        variable <- "distance"
+        DOF[i] <- x[[i]]$DOF[1,2]
+        AREA[i] <- x[[i]]$CI[1,2,'est']
+      }
       if(CLASS=="speed" || variable=="speed")
       {
         variable <- "speed"
@@ -641,6 +669,8 @@ meta.uni <- function(x,variable="area",level=0.95,level.UD=0.95,level.pop=0.95,m
   { variable <- "speed" }
   else if(CLASS=="overlap")
   { variable <- "overlap" }
+  else if(CLASS=="distance")
+  { variable <- "distance" }
 
   if(SUBPOP)
   {
@@ -704,7 +734,7 @@ meta.uni <- function(x,variable="area",level=0.95,level.UD=0.95,level.pop=0.95,m
     VAR.UNITS <- "time"
     VAR.NAME <- "Velocity timescale"
   }
-  else if(variable=="overlap")
+  else if(variable %in% c("overlap","distance"))
   {
     VAR.UNITS <- "dissimilarity"
     VAR.NAME <- "Distance"
@@ -731,7 +761,9 @@ meta.uni <- function(x,variable="area",level=0.95,level.UD=0.95,level.pop=0.95,m
     if(length(UNITS$name)) { xlab <- paste0(xlab," (",UNITS$name,")") }
     if(variable=="area") { xlab <- paste0(100*level.UD,"% ",xlab) }
     # base layer plot
-    plot(range(PLOT),c(1,N+mean),col=grDevices::rgb(1,1,1,0),xlab=xlab,ylab=NA,yaxt="n",...)
+    RANGE <- range(PLOT[PLOT<Inf])
+    RANGE[2] <- min(RANGE[2],10*PLOT[3,N+1])
+    plot(RANGE,c(1,N+mean),col=grDevices::rgb(1,1,1,0),xlab=xlab,ylab=NA,yaxt="n",...)
 
     # 2nd attempt to fix long labels # still not working, but better than nothing
     CEX.AXIS <- graphics::par("cex.axis")
@@ -783,6 +815,8 @@ meta.uni <- function(x,variable="area",level=0.95,level.UD=0.95,level.pop=0.95,m
     DEN <- sapply(RESULTS,function(R){R$CI[2,2]})
     DVAR <- sapply(RESULTS,function(R){R$VAR[2]})
 
+    dof <- pmax(2*NUM^2/NVAR,1)
+
     CI <- array(1,c(N,N,3))
     PV <- array(1,c(N,N))
     dimnames(CI) <- list(paste0(ID,"/"),paste0("/",ID),NAMES.CI)
@@ -792,7 +826,7 @@ meta.uni <- function(x,variable="area",level=0.95,level.UD=0.95,level.pop=0.95,m
       for(j in (1:N)[-i]) # diagonal == 1/1
       {
         CI[i,j,] <- F.CI(NUM[i],NVAR[i],DEN[j],DVAR[j],level=level)
-        PV[i,j] <- stats::pf(NUM[i]/NUM[j],2*NUM[i]^2/NVAR[i],2*NUM[j]^2/NVAR[j],lower.tail=NUM[i]<NUM[j])
+        PV[i,j] <- stats::pf(NUM[i]/NUM[j],dof[i],dof[j],lower.tail=NUM[i]<NUM[j])
       }
     }
 
