@@ -208,8 +208,8 @@ line.boxer <- function(dp,p0=dp[,1],lower=-Inf,upper=Inf,period=F,period.max=1/2
 QuadSolve <- function(P0,P1,P2,DIR,F0,F1,F2)
 {
   # convert back to displacements
-  P1 <- P1 - P0
-  P2 <- P2 - P0
+  P1 <- P1 - c(P0)
+  P2 <- P2 - c(P0)
   # signed magnitudes of displacement vectors
   # P1, P2, DIR columns will always be parallel
   DIR <- cbind(DIR)
@@ -357,9 +357,9 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
   DEBUG <- FALSE # can be overridden in control
   PMAP <- TRUE # periodic parameters are mapped locally: (-period/2,+period/2) -> (-Inf,Inf) during search steps
   # check complains about visible bindings
-  fnscale <- parscale <- maxit <- precision <- trace <- cores <- hessian <- covariance <- NULL
+  fnscale <- parscale <- rescale <- maxit <- precision <- trace <- cores <- hessian <- covariance <- NULL
   # fix default control arguments
-  default <- list(fnscale=1,parscale=pmin(abs(par),abs(par-lower),abs(upper-par)),maxit=100,trace=FALSE,precision=NULL,cores=1,hessian=NULL,covariance=NULL,stages=NULL)
+  default <- list(fnscale=1,parscale=pmin(abs(par),abs(par-lower),abs(upper-par)),maxit=100,trace=FALSE,precision=NULL,cores=1,hessian=NULL,covariance=NULL,stages=NULL,rescale=FALSE)
   control <- replace(default,names(control),control)
   # check does not like attach
   NAMES <- names(control)
@@ -409,7 +409,7 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
   if(!is.null(covariance))
   {
     covariance <- t(t(covariance/parscale)/parscale)
-    TEST <- eigen(covariance,only.values=TRUE)$values
+    TEST <- eigen(covariance)$values
     TEST <- any(TEST<=TOL[1]) || any(TEST>=1/TOL[1]) || min(TEST)/max(TEST)<=TOL[1]
     if(TEST) { covariance <- NULL }
   }
@@ -417,7 +417,7 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
   if(!is.null(hessian))
   {
     hessian <- t(t(hessian*parscale)*parscale)
-    TEST <- 1/eigen(hessian,only.values=TRUE)$values
+    TEST <- 1/eigen(hessian)$values
     TEST <- any(TEST<=TOL[1]) || any(TEST>=1/TOL[1]) || min(TEST)/max(TEST)<=TOL[1]
     if(TEST) { hessian <- NULL }
   }
@@ -544,7 +544,9 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
     # single parameter scale for each axis
     # SCL <- sqrt(abs(diag(SCL)))
     # equivalent calculation to the above, but avoids matrix-matrix multiplication
-    SCL <- sqrt(abs(c(DIR^2 %*% SCL^2)))
+    # SCL <- sqrt(abs(c(DIR^2 %*% SCL^2))) # can overflow
+    SCL <- diag(SCL,nrow=length(SCL))
+    SCL <- diag( t(DIR) %*% SCL %*% DIR )
     # sample initial points around the center for numerical differentiation
     STEP <- SCL*STEP[STAGE]
     par.step <- t(STEP*t(DIR))
@@ -611,12 +613,12 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
     }
 
     ## shift back near origin to prevent overflow
-    rescale <- reset(par*parscale)/parscale
-    if(sqrt(sum((rescale-par)^2))>DIM*.Machine$double.eps)
+    RESCALE <- reset(par*parscale)/parscale
+    if(sqrt(sum((RESCALE-par)^2))>DIM*.Machine$double.eps)
     {
       # rescale parameters
-      TEMP <- rescale
-      rescale <- nant(TEMP/par,1)
+      TEMP <- RESCALE
+      RESCALE <- nant(TEMP/par,1)
       par <- par.target <- par.old <- TEMP
       # reset gradients
       CG.RESET <- TRUE
@@ -1243,5 +1245,14 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
   RETURN$lower <- lower*parscale
   RETURN$upper <- upper*parscale
 
+  # initial parscale was bad [UNFINISHED]
+  if(rescale && ERROR>TOL.GOAL && max(abs(log2(par)))>1)
+  {
+    control$parscale <- RETURN$par
+    control$hessian <- RETURN$hessian
+    control$covariance <- RETURN$covariance
+    RETURN <- mc.optim(RETURN$par,fn=fn,lower=lower,upper=upper,period=period,reset=reset,control=control,...)
+  }
+
   return(RETURN)
 }
