@@ -36,9 +36,20 @@ project <- function(x,from=DATUM,to=DATUM)
 {
   if(to==from) { return(x) }
 
-  x <- sp::SpatialPoints(x,proj4string=sp::CRS(from))
-  x <- sp::spTransform(x,sp::CRS(to))
-  x <- sp::coordinates(x)
+  # x <- sp::SpatialPoints(x,proj4string=sp::CRS(from))
+  # x <- sp::spTransform(x,sp::CRS(to))
+  # x <- sp::coordinates(x)
+
+  x <- data.frame(x) # super annoying
+  from <- sf::st_crs(from)
+  to <- sf::st_crs(to)
+
+  x <- sf::st_as_sf(x,coords=1:2,crs=from)
+  x <- sf::st_transform(x,crs=to)
+  x <- sf::st_coordinates(x)
+
+  colnames(x) <- c('x','y')
+
   return(x)
 }
 
@@ -77,7 +88,7 @@ setMethod('projection<-', signature(x='list'), `projection<-.list`)
   }
 
   # convert to PROJ4 format if location
-  value <- format.projection(value)
+  value <- format_projection(value)
 
   NAMES <- names(x)
   n <- nrow(x)
@@ -175,7 +186,7 @@ rotate.north <- function(u,heading)
 
 
 # put projection into character format
-format.projection <- function(proj,datum="WGS84")
+format_projection <- function(proj,datum="WGS84")
 {
   if(class(proj)[1]=="CRS")
   { proj <- as.character(proj) }
